@@ -25,19 +25,20 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include "app/logging.h"
 #include "archiveinterface.h"
-#include <kdebug.h>
 #include <kfileitem.h>
 
-#include <QFileInfo>
+#include <QDebug>
 #include <QDir>
+#include <QFileInfo>
 
 namespace Kerfuffle
 {
 ReadOnlyArchiveInterface::ReadOnlyArchiveInterface(QObject *parent, const QVariantList & args)
         : QObject(parent), m_waitForFinishedSignal(false)
 {
-    kDebug();
+    qCDebug(KERFUFFLE) << "Created read-only interface for" << args.first().toString();
     m_filename = args.first().toString();
 }
 
@@ -65,6 +66,11 @@ void ReadOnlyArchiveInterface::setPassword(const QString &password)
     m_password = password;
 }
 
+void ReadOnlyArchiveInterface::setHeaderEncryptionEnabled(bool enabled)
+{
+    m_isHeaderEncryptionEnabled = enabled;
+}
+
 QString ReadOnlyArchiveInterface::password() const
 {
     return m_password;
@@ -91,6 +97,7 @@ bool ReadOnlyArchiveInterface::doResume()
 ReadWriteArchiveInterface::ReadWriteArchiveInterface(QObject *parent, const QVariantList & args)
         : ReadOnlyArchiveInterface(parent, args)
 {
+    qCDebug(KERFUFFLE) << "Created read-write interface for" << args.first().toString();
 }
 
 ReadWriteArchiveInterface::~ReadWriteArchiveInterface()
@@ -107,6 +114,11 @@ void ReadOnlyArchiveInterface::setWaitForFinishedSignal(bool value)
     m_waitForFinishedSignal = value;
 }
 
+bool ReadOnlyArchiveInterface::isHeaderEncryptionEnabled() const
+{
+    return m_isHeaderEncryptionEnabled;
+}
+
 bool ReadWriteArchiveInterface::isReadOnly() const
 {
     QFileInfo fileInfo(filename());
@@ -119,4 +131,4 @@ bool ReadWriteArchiveInterface::isReadOnly() const
 
 } // namespace Kerfuffle
 
-#include "archiveinterface.moc"
+
